//
// Copyright 2022 The Sigstore Authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package jar

import (
	"testing"

	"github.com/sigstore/rekor/pkg/generated/models"
)

func FuzzJarUnmarshal(f *testing.F) {
	f.Fuzz(func(t *testing.T, raw []byte) {
		m := &models.Jar{}
		err := m.UnmarshalJSON(raw)
		if err != nil {
			t.Skip()
		}
		brt := New()
		brt.UnmarshalEntry(m)
	})
}
